@@ -14,8 +14,7 @@
 #    under the License.
 
 from oslo_utils import encodeutils
-import six
-from six.moves.urllib import parse
+import urllib.parse
 import warlock
 
 from glanceclient.common import utils
@@ -37,6 +36,7 @@ class NamespaceController(object):
         return warlock.model_factory(schema.raw(),
                                      base_class=schemas.SchemaBasedModel)
 
+    @utils.add_req_id_to_object()
     def create(self, **kwargs):
         """Create a namespace.
 
@@ -50,7 +50,7 @@ class NamespaceController(object):
 
         resp, body = self.http_client.post(url, data=namespace)
         body.pop('self', None)
-        return self.model(**body)
+        return self.model(**body), resp
 
     def update(self, namespace_name, **kwargs):
         """Update a namespace.
@@ -59,7 +59,7 @@ class NamespaceController(object):
         :param kwargs: Unpacked namespace object.
         """
         namespace = self.get(namespace_name)
-        for (key, value) in six.iteritems(kwargs):
+        for (key, value) in kwargs.items():
             try:
                 setattr(namespace, key, value)
             except warlock.InvalidOperation as e:
@@ -71,24 +71,37 @@ class NamespaceController(object):
             if elem in namespace:
                 del namespace[elem]
 
-        url = '/v2/metadefs/namespaces/{0}'.format(namespace_name)
-        self.http_client.put(url, data=namespace)
-
-        return self.get(namespace.namespace)
+        url = '/v2/metadefs/namespaces/%(namespace)s' % {
+            'namespace': namespace_name}
+        # Pass the original wrapped value to http client.
+        resp, _ = self.http_client.put(url, data=namespace.wrapped)
+        # Get request id from `put` request so it can be passed to the
+        #  following `get` call
+        req_id_hdr = {
+            'x-openstack-request-id': utils._extract_request_id(resp)
+        }
+        return self._get(namespace.namespace, header=req_id_hdr)
 
     def get(self, namespace, **kwargs):
+        return self._get(namespace, **kwargs)
+
+    @utils.add_req_id_to_object()
+    def _get(self, namespace, header=None, **kwargs):
         """Get one namespace."""
-        query_params = parse.urlencode(kwargs)
+        query_params = urllib.parse.urlencode(kwargs)
         if kwargs:
             query_params = '?%s' % query_params
 
-        url = '/v2/metadefs/namespaces/{0}{1}'.format(namespace, query_params)
-        resp, body = self.http_client.get(url)
+        url = '/v2/metadefs/namespaces/%(namespace)s%(query_params)s' % {
+            'namespace': namespace, 'query_params': query_params}
+        header = header or {}
+        resp, body = self.http_client.get(url, headers=header)
         # NOTE(bcwaldon): remove 'self' for now until we have an elegant
         # way to pass it into the model constructor without conflict
         body.pop('self', None)
-        return self.model(**body)
+        return self.model(**body), resp
 
+    @utils.add_req_id_to_generator()
     def list(self, **kwargs):
         """Retrieve a listing of Namespace objects.
 
@@ -117,7 +130,7 @@ class NamespaceController(object):
                 # an elegant way to pass it into the model constructor
                 # without conflict.
                 namespace.pop('self', None)
-                yield self.model(**namespace)
+                yield self.model(**namespace), resp
                 # NOTE(zhiyan): In order to resolve the performance issue
                 # of JSON schema validation for image listing case, we
                 # don't validate each image entry but do it only on first
@@ -132,8 +145,8 @@ class NamespaceController(object):
             except KeyError:
                 return
             else:
-                for namespace in paginate(next_url):
-                    yield namespace
+                for namespace, resp in paginate(next_url):
+                    yield namespace, resp
 
         filters = kwargs.get('filters', {})
         filters = {} if filters is None else filters
@@ -162,21 +175,24 @@ class NamespaceController(object):
                 raise ValueError('sort_dir must be one of the following: %s.'
                                  % ', '.join(SORT_DIR_VALUES))
 
-        for param, value in six.iteritems(filters):
+        for param, value in filters.items():
             if isinstance(value, list):
                 filters[param] = encodeutils.safe_encode(','.join(value))
-            elif isinstance(value, six.string_types):
+            elif isinstance(value, str):
                 filters[param] = encodeutils.safe_encode(value)
 
-        url = '/v2/metadefs/namespaces?%s' % parse.urlencode(filters)
+        url = '/v2/metadefs/namespaces?%s' % urllib.parse.urlencode(filters)
 
-        for namespace in paginate(url):
-            yield namespace
+        for namespace, resp in paginate(url):
+            yield namespace, resp
 
+    @utils.add_req_id_to_object()
     def delete(self, namespace):
         """Delete a namespace."""
-        url = '/v2/metadefs/namespaces/{0}'.format(namespace)
-        self.http_client.delete(url)
+        url = '/v2/metadefs/namespaces/%(namespace)s' % {
+            'namespace': namespace}
+        resp, body = self.http_client.delete(url)
+        return (resp, body), resp
 
 
 class ResourceTypeController(object):
@@ -190,6 +206,7 @@ class ResourceTypeController(object):
         return warlock.model_factory(schema.raw(),
                                      base_class=schemas.SchemaBasedModel)
 
+    @utils.add_req_id_to_object()
     def associate(self, namespace, **kwargs):
         """Associate a resource type with a namespace."""
         try:
@@ -197,18 +214,22 @@ class ResourceTypeController(object):
         except (warlock.InvalidOperation, ValueError) as e:
             raise TypeError(encodeutils.exception_to_unicode(e))
 
-        url = '/v2/metadefs/namespaces/{0}/resource_types'.format(namespace,
-                                                                  res_type)
+        url = '/v2/metadefs/namespaces/%(namespace)s/resource_types' % {
+            'namespace': namespace}
         resp, body = self.http_client.post(url, data=res_type)
         body.pop('self', None)
-        return self.model(**body)
+        return self.model(**body), resp
 
+    @utils.add_req_id_to_object()
     def deassociate(self, namespace, resource):
         """Deassociate a resource type with a namespace."""
-        url = '/v2/metadefs/namespaces/{0}/resource_types/{1}'. \
-            format(namespace, resource)
-        self.http_client.delete(url)
+        url = ('/v2/metadefs/namespaces/%(namespace)s/'
+               'resource_types/%(resource)s') % {
+            'namespace': namespace, 'resource': resource}
+        resp, body = self.http_client.delete(url)
+        return (resp, body), resp
 
+    @utils.add_req_id_to_generator()
     def list(self):
         """Retrieve a listing of available resource types.
 
@@ -218,14 +239,16 @@ class ResourceTypeController(object):
         url = '/v2/metadefs/resource_types'
         resp, body = self.http_client.get(url)
         for resource_type in body['resource_types']:
-            yield self.model(**resource_type)
+            yield self.model(**resource_type), resp
 
+    @utils.add_req_id_to_generator()
     def get(self, namespace):
-        url = '/v2/metadefs/namespaces/{0}/resource_types'.format(namespace)
+        url = '/v2/metadefs/namespaces/%(namespace)s/resource_types' % {
+            'namespace': namespace}
         resp, body = self.http_client.get(url)
         body.pop('self', None)
         for resource_type in body['resource_type_associations']:
-            yield self.model(**resource_type)
+            yield self.model(**resource_type), resp
 
 
 class PropertyController(object):
@@ -239,6 +262,7 @@ class PropertyController(object):
         return warlock.model_factory(schema.raw(),
                                      base_class=schemas.SchemaBasedModel)
 
+    @utils.add_req_id_to_object()
     def create(self, namespace, **kwargs):
         """Create a property.
 
@@ -250,11 +274,11 @@ class PropertyController(object):
         except (warlock.InvalidOperation, ValueError) as e:
             raise TypeError(encodeutils.exception_to_unicode(e))
 
-        url = '/v2/metadefs/namespaces/{0}/properties'.format(namespace)
-
+        url = '/v2/metadefs/namespaces/%(namespace)s/properties' % {
+            'namespace': namespace}
         resp, body = self.http_client.post(url, data=prop)
         body.pop('self', None)
-        return self.model(**body)
+        return self.model(**body), resp
 
     def update(self, namespace, prop_name, **kwargs):
         """Update a property.
@@ -270,43 +294,63 @@ class PropertyController(object):
             except warlock.InvalidOperation as e:
                 raise TypeError(encodeutils.exception_to_unicode(e))
 
-        url = '/v2/metadefs/namespaces/{0}/properties/{1}'.format(namespace,
-                                                                  prop_name)
-        self.http_client.put(url, data=prop)
+        url = ('/v2/metadefs/namespaces/%(namespace)s/'
+               'properties/%(prop_name)s') % {
+            'namespace': namespace, 'prop_name': prop_name}
+        # Pass the original wrapped value to http client.
+        resp, _ = self.http_client.put(url, data=prop.wrapped)
+        # Get request id from `put` request so it can be passed to the
+        #  following `get` call
+        req_id_hdr = {
+            'x-openstack-request-id': utils._extract_request_id(resp)}
 
-        return self.get(namespace, prop.name)
+        return self._get(namespace, prop.name, req_id_hdr)
 
     def get(self, namespace, prop_name):
-        url = '/v2/metadefs/namespaces/{0}/properties/{1}'.format(namespace,
-                                                                  prop_name)
-        resp, body = self.http_client.get(url)
+        return self._get(namespace, prop_name)
+
+    @utils.add_req_id_to_object()
+    def _get(self, namespace, prop_name, header=None):
+        url = ('/v2/metadefs/namespaces/%(namespace)s/'
+               'properties/%(prop_name)s') % {
+            'namespace': namespace, 'prop_name': prop_name}
+        header = header or {}
+        resp, body = self.http_client.get(url, headers=header)
         body.pop('self', None)
         body['name'] = prop_name
-        return self.model(**body)
+        return self.model(**body), resp
 
+    @utils.add_req_id_to_generator()
     def list(self, namespace, **kwargs):
         """Retrieve a listing of metadata properties.
 
         :returns: generator over list of objects
         """
-        url = '/v2/metadefs/namespaces/{0}/properties'.format(namespace)
+        url = '/v2/metadefs/namespaces/%(namespace)s/properties' % {
+            'namespace': namespace}
 
         resp, body = self.http_client.get(url)
 
         for key, value in body['properties'].items():
             value['name'] = key
-            yield self.model(value)
+            yield self.model(value), resp
 
+    @utils.add_req_id_to_object()
     def delete(self, namespace, prop_name):
         """Delete a property."""
-        url = '/v2/metadefs/namespaces/{0}/properties/{1}'.format(namespace,
-                                                                  prop_name)
-        self.http_client.delete(url)
+        url = ('/v2/metadefs/namespaces/%(namespace)s/'
+               'properties/%(prop_name)s') % {
+            'namespace': namespace, 'prop_name': prop_name}
+        resp, body = self.http_client.delete(url)
+        return (resp, body), resp
 
+    @utils.add_req_id_to_object()
     def delete_all(self, namespace):
         """Delete all properties in a namespace."""
-        url = '/v2/metadefs/namespaces/{0}/properties'.format(namespace)
-        self.http_client.delete(url)
+        url = '/v2/metadefs/namespaces/%(namespace)s/properties' % {
+            'namespace': namespace}
+        resp, body = self.http_client.delete(url)
+        return (resp, body), resp
 
 
 class ObjectController(object):
@@ -320,6 +364,7 @@ class ObjectController(object):
         return warlock.model_factory(schema.raw(),
                                      base_class=schemas.SchemaBasedModel)
 
+    @utils.add_req_id_to_object()
     def create(self, namespace, **kwargs):
         """Create an object.
 
@@ -331,11 +376,12 @@ class ObjectController(object):
         except (warlock.InvalidOperation, ValueError) as e:
             raise TypeError(encodeutils.exception_to_unicode(e))
 
-        url = '/v2/metadefs/namespaces/{0}/objects'.format(namespace)
+        url = '/v2/metadefs/namespaces/%(namespace)s/objects' % {
+            'namespace': namespace}
 
         resp, body = self.http_client.post(url, data=obj)
         body.pop('self', None)
-        return self.model(**body)
+        return self.model(**body), resp
 
     def update(self, namespace, object_name, **kwargs):
         """Update an object.
@@ -357,40 +403,60 @@ class ObjectController(object):
             if elem in obj:
                 del obj[elem]
 
-        url = '/v2/metadefs/namespaces/{0}/objects/{1}'.format(namespace,
-                                                               object_name)
-        self.http_client.put(url, data=obj)
+        url = ('/v2/metadefs/namespaces/%(namespace)s/'
+               'objects/%(object_name)s') % {
+            'namespace': namespace, 'object_name': object_name}
+        # Pass the original wrapped value to http client.
+        resp, _ = self.http_client.put(url, data=obj.wrapped)
+        # Get request id from `put` request so it can be passed to the
+        #  following `get` call
+        req_id_hdr = {
+            'x-openstack-request-id': utils._extract_request_id(resp)}
 
-        return self.get(namespace, obj.name)
+        return self._get(namespace, obj.name, req_id_hdr)
 
     def get(self, namespace, object_name):
-        url = '/v2/metadefs/namespaces/{0}/objects/{1}'.format(namespace,
-                                                               object_name)
-        resp, body = self.http_client.get(url)
-        body.pop('self', None)
-        return self.model(**body)
+        return self._get(namespace, object_name)
 
+    @utils.add_req_id_to_object()
+    def _get(self, namespace, object_name, header=None):
+        url = ('/v2/metadefs/namespaces/%(namespace)s/'
+               'objects/%(object_name)s') % {
+            'namespace': namespace, 'object_name': object_name}
+        header = header or {}
+        resp, body = self.http_client.get(url, headers=header)
+        body.pop('self', None)
+        return self.model(**body), resp
+
+    @utils.add_req_id_to_generator()
     def list(self, namespace, **kwargs):
         """Retrieve a listing of metadata objects.
 
         :returns: generator over list of objects
         """
-        url = '/v2/metadefs/namespaces/{0}/objects'.format(namespace,)
+        url = '/v2/metadefs/namespaces/%(namespace)s/objects' % {
+            'namespace': namespace}
         resp, body = self.http_client.get(url)
 
         for obj in body['objects']:
-            yield self.model(obj)
+            yield self.model(obj), resp
 
+    @utils.add_req_id_to_object()
     def delete(self, namespace, object_name):
         """Delete an object."""
-        url = '/v2/metadefs/namespaces/{0}/objects/{1}'.format(namespace,
-                                                               object_name)
-        self.http_client.delete(url)
+        url = ('/v2/metadefs/namespaces/%(namespace)s/'
+               'objects/%(object_name)s') % {
+            'namespace': namespace, 'object_name': object_name}
+        resp, body = self.http_client.delete(url)
+        return (resp, body), resp
 
+    @utils.add_req_id_to_object()
     def delete_all(self, namespace):
         """Delete all objects in a namespace."""
-        url = '/v2/metadefs/namespaces/{0}/objects'.format(namespace)
-        self.http_client.delete(url)
+        url = '/v2/metadefs/namespaces/%(namespace)s/objects' % {
+            'namespace': namespace}
+        resp, body = self.http_client.delete(url)
+        return (resp, body), resp
 
 
 class TagController(object):
@@ -404,6 +470,7 @@ class TagController(object):
         return warlock.model_factory(schema.raw(),
                                      base_class=schemas.SchemaBasedModel)
 
+    @utils.add_req_id_to_object()
     def create(self, namespace, tag_name):
         """Create a tag.
 
@@ -411,13 +478,14 @@ class TagController(object):
         :param tag_name: The name of the new tag to create.
         """
 
-        url = ('/v2/metadefs/namespaces/{0}/tags/{1}'.format(namespace,
-                                                             tag_name))
+        url = '/v2/metadefs/namespaces/%(namespace)s/tags/%(tag_name)s' % {
+            'namespace': namespace, 'tag_name': tag_name}
 
         resp, body = self.http_client.post(url)
         body.pop('self', None)
-        return self.model(**body)
+        return self.model(**body), resp
 
+    @utils.add_req_id_to_generator()
     def create_multiple(self, namespace, **kwargs):
         """Create the list of tags.
 
@@ -435,12 +503,13 @@ class TagController(object):
                 raise TypeError(encodeutils.exception_to_unicode(e))
         tags = {'tags': md_tag_list}
 
-        url = '/v2/metadefs/namespaces/{0}/tags'.format(namespace)
+        url = '/v2/metadefs/namespaces/%(namespace)s/tags' % {
+            'namespace': namespace}
 
         resp, body = self.http_client.post(url, data=tags)
         body.pop('self', None)
         for tag in body['tags']:
-            yield self.model(tag)
+            yield self.model(tag), resp
 
     def update(self, namespace, tag_name, **kwargs):
         """Update a tag.
@@ -462,37 +531,54 @@ class TagController(object):
             if elem in tag:
                 del tag[elem]
 
-        url = '/v2/metadefs/namespaces/{0}/tags/{1}'.format(namespace,
-                                                            tag_name)
-        self.http_client.put(url, data=tag)
+        url = '/v2/metadefs/namespaces/%(namespace)s/tags/%(tag_name)s' % {
+            'namespace': namespace, 'tag_name': tag_name}
+        # Pass the original wrapped value to http client.
+        resp, _ = self.http_client.put(url, data=tag.wrapped)
+        # Get request id from `put` request so it can be passed to the
+        #  following `get` call
+        req_id_hdr = {
+            'x-openstack-request-id': utils._extract_request_id(resp)}
 
-        return self.get(namespace, tag.name)
+        return self._get(namespace, tag.name, req_id_hdr)
 
     def get(self, namespace, tag_name):
-        url = '/v2/metadefs/namespaces/{0}/tags/{1}'.format(namespace,
-                                                            tag_name)
-        resp, body = self.http_client.get(url)
-        body.pop('self', None)
-        return self.model(**body)
+        return self._get(namespace, tag_name)
 
+    @utils.add_req_id_to_object()
+    def _get(self, namespace, tag_name, header=None):
+        url = '/v2/metadefs/namespaces/%(namespace)s/tags/%(tag_name)s' % {
+            'namespace': namespace, 'tag_name': tag_name}
+        header = header or {}
+        resp, body = self.http_client.get(url, headers=header)
+        body.pop('self', None)
+        return self.model(**body), resp
+
+    @utils.add_req_id_to_generator()
     def list(self, namespace, **kwargs):
         """Retrieve a listing of metadata tags.
 
         :returns: generator over list of tags.
         """
-        url = '/v2/metadefs/namespaces/{0}/tags'.format(namespace)
+        url = '/v2/metadefs/namespaces/%(namespace)s/tags' % {
+            'namespace': namespace}
         resp, body = self.http_client.get(url)
 
         for tag in body['tags']:
-            yield self.model(tag)
+            yield self.model(tag), resp
 
+    @utils.add_req_id_to_object()
     def delete(self, namespace, tag_name):
         """Delete a tag."""
-        url = '/v2/metadefs/namespaces/{0}/tags/{1}'.format(namespace,
-                                                            tag_name)
-        self.http_client.delete(url)
+        url = '/v2/metadefs/namespaces/%(namespace)s/tags/%(tag_name)s' % {
+            'namespace': namespace, 'tag_name': tag_name}
+        resp, body = self.http_client.delete(url)
+        return (resp, body), resp
 
+    @utils.add_req_id_to_object()
     def delete_all(self, namespace):
         """Delete all tags in a namespace."""
-        url = '/v2/metadefs/namespaces/{0}/tags'.format(namespace)
-        self.http_client.delete(url)
+        url = '/v2/metadefs/namespaces/%(namespace)s/tags' % {
+            'namespace': namespace}
+        resp, body = self.http_client.delete(url)
+        return (resp, body), resp
